@@ -151,7 +151,7 @@ evaluateNode (le * node)
   return value;
 }
 
-DEFUN_HIDDEN ("setq", setq)
+DEFUN_NONINTERACTIVE ("setq", setq)
 /*+
 (setq [sym val]...)
 
@@ -177,7 +177,7 @@ The values val are expressions; they are evaluated.
         }
     }
 
-  return newvalue;
+  ok = newvalue;
 }
 END_DEFUN
 
@@ -200,11 +200,11 @@ execute_with_uniarg (bool undo, int uniarg, int (*forward) (void), int (*backwar
       uniarg = -uniarg;
     }
   if (undo)
-    undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+    undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
   for (uni = 0; ret && uni < uniarg; ++uni)
     ret = func ();
   if (undo)
-    undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+    undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 
   return bool_to_lisp (ret);
 }
@@ -270,9 +270,9 @@ minibuf_read_function_name (const char *fmt, ...)
 
   for (i = 0; i < fentry_table_size; ++i)
     if (fentry_table[i].interactive)
-      gl_sortedlist_add (cp->completions, completion_strcmp,
+      gl_sortedlist_add (get_completion_completions (cp), completion_strcmp,
                          xstrdup (fentry_table[i].name));
-  add_macros_to_list (cp->completions, completion_strcmp);
+  add_macros_to_list (get_completion_completions (cp), completion_strcmp);
 
   va_start (ap, fmt);
   ms = minibuf_vread_completion (fmt, "", cp, functions_history,
