@@ -80,10 +80,14 @@ make_buffer_flags (Buffer * bp, int iscurrent)
   static char buf[4];
 
   buf[0] = iscurrent ? '.' : ' ';
-  buf[1] = (bp->flags & BFLAG_MODIFIED) ? '*' : ' ';
-  /* Display the readonly flag if it is set or the buffer is
-     the current buffer, i.e. the `*Buffer List*' buffer. */
-  buf[2] = (bp->flags & BFLAG_READONLY || bp == cur_bp) ? '%' : ' ';
+  buf[1] = get_buffer_modified (bp) ? '*' : ' ';
+  /*
+   * Display the readonly flag if it is set or the buffer is the
+   * `*Buffer List*' buffer.  (The readonly flag is not set when we are
+   * called, as otherwise we wouldn't be able to write to the
+   * buffer!)
+   */
+  buf[2] = (get_buffer_readonly (bp) || bp == cur_bp) ? '%' : ' ';
   buf[3] = '\0';
 
   return buf;
@@ -94,7 +98,7 @@ make_buffer_modeline (Buffer * bp)
 {
   astr as = astr_new ();
 
-  if (bp->flags & BFLAG_AUTOFILL)
+  if (get_buffer_autofill (bp))
     astr_cat_cstr (as, " Fill");
 
   return as;
@@ -105,17 +109,13 @@ make_buffer_modeline (Buffer * bp)
  * if a cut is needed.
  */
 static astr
-shorten_string (char *s, int maxlen)
+shorten_string (astr as, int maxlen)
 {
-  astr as = astr_new ();
-  int len = strlen (s);
-
-  if (len <= maxlen)
-    astr_cpy_cstr (as, s);
-  else
+  int len = astr_len (as);
+  if (len > maxlen)
     {
-      astr_cpy_cstr (as, "...");
-      astr_cat_cstr (as, s + len - maxlen + 3);
+      astr_replace_cstr (as, 0, 0, "...");
+      astr_truncate (as, len - maxlen + 3);
     }
 
   return as;
@@ -126,16 +126,16 @@ print_buf (Buffer * old_bp, Buffer * bp)
 {
   astr mode = make_buffer_modeline (bp);
 
-  if (bp->name[0] == ' ')
+  if (get_buffer_name (bp)[0] == ' ')
     return;
 
   bprintf ("%3s %-16s %6u  %-13s",
            make_buffer_flags (bp, old_bp == bp),
-           bp->name, calculate_buffer_size (bp), astr_cstr (mode));
+           get_buffer_name (bp), calculate_buffer_size (bp), astr_cstr (mode));
   astr_delete (mode);
-  if (bp->filename != NULL)
+  if (get_buffer_filename (bp) != NULL)
     {
-      astr shortname = shorten_string (bp->filename, 40);
+      astr shortname = shorten_string (compact_path (astr_new_cstr (get_buffer_filename (bp))), 40);
       insert_astr (shortname);
       astr_delete (shortname);
     }
@@ -155,18 +155,28 @@ write_temp_buffer (const char *name, bool show, void (*func) (va_list ap), ...)
     set_current_window (wp);
   else
     {
+      Buffer *bp = find_buffer (name);
       if (show)
         set_current_window (popup_window ());
-      switch_to_buffer (find_buffer (name, true));
+      if (bp == NULL)
+        {
+          bp = buffer_new ();
+          set_buffer_name (bp, name);
+        }
+      switch_to_buffer (bp);
     }
 
-  /* Remove all the content of that buffer. */
-  new_bp = create_buffer (cur_bp->name);
+  /* Remove the contents of that buffer. */
+  new_bp = buffer_new ();
+  set_buffer_name (new_bp, get_buffer_name (cur_bp));
   kill_buffer (cur_bp);
-  cur_bp = cur_wp->bp = new_bp;
+  cur_bp = new_bp;
+  set_window_bp (cur_wp, cur_bp);
 
-  /* Make the buffer like a temporary one. */
-  cur_bp->flags = BFLAG_NEEDNAME | BFLAG_NOSAVE | BFLAG_NOUNDO;
+  /* Make the buffer a temporary one. */
+  set_buffer_needname (cur_bp, true);
+  set_buffer_noundo (cur_bp, true);
+  set_buffer_nosave (cur_bp, true);
   set_temporary_buffer (cur_bp);
 
   /* Use the "callback" routine. */
@@ -175,7 +185,7 @@ write_temp_buffer (const char *name, bool show, void (*func) (va_list ap), ...)
   va_end (ap);
 
   gotobob ();
-  cur_bp->flags |= BFLAG_READONLY;
+  set_buffer_readonly (cur_bp, true);
 
   /* Restore old current window. */
   set_current_window (old_wp);
@@ -195,17 +205,17 @@ write_buffers_list (va_list ap)
   bprintf (" -- ------           ----    ----         ----\n");
 
   /* Print buffers. */
-  bp = old_wp->bp;
+  bp = get_window_bp (old_wp);
   do
     {
-      /* Print all buffers less this one (the *Buffer List*). */
+      /* Print all buffers except this one (the *Buffer List*). */
       if (cur_bp != bp)
-        print_buf (old_wp->bp, bp);
-      bp = bp->next;
+        print_buf (get_window_bp (old_wp), bp);
+      bp = get_buffer_next (bp);
       if (bp == NULL)
         bp = head_bp;
     }
-  while (bp != old_wp->bp);
+  while (bp != get_window_bp (old_wp));
 }
 
 DEFUN ("list-buffers", list_buffers)
@@ -233,7 +243,7 @@ At the end of a line, such characters extend the line.
 is supposed to make it easier to insert characters when necessary.
 +*/
 {
-  cur_bp->flags ^= BFLAG_OVERWRITE;
+  set_buffer_overwrite (cur_bp, !get_buffer_overwrite (cur_bp));
 }
 END_DEFUN
 
@@ -242,7 +252,7 @@ DEFUN ("toggle-read-only", toggle_read_only)
 Change whether this buffer is visiting its file read-only.
 +*/
 {
-  cur_bp->flags ^= BFLAG_READONLY;
+  set_buffer_readonly (cur_bp, !get_buffer_readonly (cur_bp));
 }
 END_DEFUN
 
@@ -253,7 +263,7 @@ In Auto Fill mode, inserting a space at a column beyond `fill-column'
 automatically breaks the line at a previous space.
 +*/
 {
-  cur_bp->flags ^= BFLAG_AUTOFILL;
+  set_buffer_autofill (cur_bp, !get_buffer_autofill (cur_bp));
 }
 END_DEFUN
 
@@ -265,16 +275,16 @@ Just C-u as argument means to use the current column.
 +*/
 {
   long fill_col = (lastflag & FLAG_UNIARG_EMPTY) ?
-    cur_bp->pt.o : (unsigned long) uniarg;
+    get_buffer_pt (cur_bp).o : (unsigned long) uniarg;
   char *buf;
 
   if (!(lastflag & FLAG_SET_UNIARG) && arglist == NULL)
     {
-      fill_col = minibuf_read_number ("Set fill-column to (default %d): ", cur_bp->pt.o);
+      fill_col = minibuf_read_number ("Set fill-column to (default %d): ", get_buffer_pt (cur_bp).o);
       if (fill_col == LONG_MAX)
         return leNIL;
       else if (fill_col == LONG_MAX - 1)
-        fill_col = cur_bp->pt.o;
+        fill_col = get_buffer_pt (cur_bp).o;
     }
 
   if (arglist)
@@ -329,13 +339,17 @@ DEFUN ("exchange-point-and-mark", exchange_point_and_mark)
 Put the mark where point is now, and point where the mark is now.
 +*/
 {
-  if (!cur_bp->mark)
+  Point tmp;
+
+  if (get_buffer_mark (cur_bp) == NULL)
     {
       minibuf_error ("No mark set in this buffer");
       return leNIL;
     }
 
-  swap_point (&cur_bp->pt, &cur_bp->mark->pt);
+  tmp = get_buffer_pt (cur_bp);
+  set_buffer_pt (cur_bp, get_marker_pt (get_buffer_mark (cur_bp)));
+  set_marker_pt (get_buffer_mark (cur_bp), tmp);
 
   /* In transient-mark-mode we must reactivate the mark.  */
   if (transient_mark_mode ())
@@ -579,14 +593,14 @@ edit_tab_line (Line ** lp, size_t lineno, size_t offset, size_t size,
 
   src = (char *) xzalloc (size + 1);
   dest = (char *) xzalloc (size * t + 1);
-  strncpy (src, astr_cstr ((*lp)->text) + offset, size);
+  strncpy (src, astr_cstr (get_line_text (*lp)) + offset, size);
   src[size] = '\0';
 
   /* Get offset's column.  */
   col = 0;
   for (i = 0; i < offset; i++)
     {
-      if (*astr_char ((*lp)->text, i) == '\t')
+      if (astr_get (get_line_text (*lp), i) == '\t')
         col |= t - 1;
       ++col;
     }
@@ -611,48 +625,51 @@ edit_tab_line (Line ** lp, size_t lineno, size_t offset, size_t size,
 static le *
 edit_tab_region (int action)
 {
-  Region r;
-  Line *lp;
-  size_t lineno;
-  Marker *marker;
+  Region * rp = region_new ();
 
-  if (warn_if_readonly_buffer () || !calculate_the_region (&r))
-    return leNIL;
-
-  if (r.size == 0)
-    return leT;
-
-  marker = point_marker ();
-
-  undo_save (UNDO_START_SEQUENCE, marker->pt, 0, 0);
-  for (lp = r.start.p, lineno = r.start.n;; lp = lp->next, ++lineno)
+  if (warn_if_readonly_buffer () || !calculate_the_region (rp))
     {
-      /* First line.  */
-      if (lineno == r.start.n)
-        {
-          /* Region on a sole line. */
-          if (lineno == r.end.n)
-            edit_tab_line (&lp, lineno, r.start.o, r.size, action);
-          /* Region is multi-line. */
-          else
-            edit_tab_line (&lp, lineno, r.start.o,
-                           astr_len (lp->text) - r.start.o, action);
-        }
-      /* Last line of multi-line region. */
-      else if (lineno == r.end.n)
-        edit_tab_line (&lp, lineno, 0, r.end.o, action);
-      /* Middle line of multi-line region. */
-      else
-        edit_tab_line (&lp, lineno, 0, astr_len (lp->text), action);
-      /* Done?  */
-      if (lineno == r.end.n)
-        break;
+      free (rp);
+      return leNIL;
     }
-  cur_bp->pt = marker->pt;
-  undo_save (UNDO_END_SEQUENCE, marker->pt, 0, 0);
-  free_marker (marker);
-  deactivate_mark ();
 
+  if (get_region_size (rp) != 0)
+    {
+      Marker *m = point_marker ();
+      Line * lp;
+      size_t lineno;
+
+      undo_save (UNDO_START_SEQUENCE, get_marker_pt (m), 0, 0);
+      for (lp = get_region_start (rp).p, lineno = get_region_start (rp).n;; lp = get_line_next (lp), ++lineno)
+        {
+          /* First line.  */
+          if (lineno == get_region_start (rp).n)
+            {
+              /* Region on a sole line. */
+              if (lineno == get_region_end (rp).n)
+                edit_tab_line (&lp, lineno, get_region_start (rp).o, get_region_size (rp), action);
+              /* Region is multi-line. */
+              else
+                edit_tab_line (&lp, lineno, get_region_start (rp).o,
+                               astr_len (get_line_text (lp)) - get_region_start (rp).o, action);
+            }
+          /* Last line of multi-line region. */
+          else if (lineno == get_region_end (rp).n)
+            edit_tab_line (&lp, lineno, 0, get_region_end (rp).o, action);
+          /* Middle line of multi-line region. */
+          else
+            edit_tab_line (&lp, lineno, 0, astr_len (get_line_text (lp)), action);
+          /* Done?  */
+          if (lineno == get_region_end (rp).n)
+            break;
+        }
+      set_buffer_pt (cur_bp, get_marker_pt (m));
+      undo_save (UNDO_END_SEQUENCE, get_marker_pt (m), 0, 0);
+      free_marker (m);
+      deactivate_mark ();
+    }
+
+  free (rp);
   return leT;
 }
 
@@ -683,7 +700,7 @@ DEFUN ("back-to-indentation", back_to_indentation)
 Move point to the first non-whitespace character on this line.
 +*/
 {
-  cur_bp->pt = line_beginning_position (1);
+  set_buffer_pt (cur_bp, line_beginning_position (1));
   while (!eolp ())
     {
       if (!isspace (following_char ()))
@@ -706,6 +723,8 @@ move_word (int dir, int (*next_char) (void), int (*move_char) (void), int (*at_e
       while (!at_extreme ())
         {
           int c = next_char ();
+          Point pt;
+
           if (!ISWORDCHAR (c))
             {
               if (gotword)
@@ -713,7 +732,9 @@ move_word (int dir, int (*next_char) (void), int (*move_char) (void), int (*at_e
             }
           else
             gotword = true;
-          cur_bp->pt.o += dir;
+          pt = get_buffer_pt (cur_bp);
+          pt.o += dir;
+          set_buffer_pt (cur_bp, pt);
         }
       if (gotword)
         return true;
@@ -768,14 +789,16 @@ END_DEFUN
                                ((c == '\'') && single_quote))
 #define ISSEXPSEPARATOR(c)    (ISOPENBRACKETCHAR (c) ||	\
                                ISCLOSEBRACKETCHAR (c))
-#define PRECEDINGQUOTEDQUOTE(c) (c == '\\' \
-    && cur_bp->pt.o + 1 < astr_len (cur_bp->pt.p->text) \
-    && ((*astr_char (cur_bp->pt.p->text, cur_bp->pt.o + 1) == '\"') || \
-        (*astr_char (cur_bp->pt.p->text, cur_bp->pt.o + 1) == '\'')))
-#define FOLLOWINGQUOTEDQUOTE(c) (c == '\\' \
-    && cur_bp->pt.o + 1 < astr_len (cur_bp->pt.p->text) \
-    && ((*astr_char (cur_bp->pt.p->text, cur_bp->pt.o + 1) == '\"') || \
-        (*astr_char (cur_bp->pt.p->text, cur_bp->pt.o + 1) == '\'')))
+#define PRECEDINGQUOTEDQUOTE(c)                                         \
+  (c == '\\'                                                            \
+   && get_buffer_pt (cur_bp).o + 1 < astr_len (get_line_text (get_buffer_pt (cur_bp).p)) \
+   && ((astr_get (get_line_text (get_buffer_pt (cur_bp).p), get_buffer_pt (cur_bp).o + 1) == '\"') || \
+       (astr_get (get_line_text (get_buffer_pt (cur_bp).p), get_buffer_pt (cur_bp).o + 1) == '\'')))
+#define FOLLOWINGQUOTEDQUOTE(c)                                         \
+  (c == '\\'                                                            \
+   && get_buffer_pt (cur_bp).o + 1 < astr_len (get_line_text (get_buffer_pt (cur_bp).p)) \
+   && ((astr_get (get_line_text (get_buffer_pt (cur_bp).p), get_buffer_pt (cur_bp).o + 1) == '\"') || \
+       (astr_get (get_line_text (get_buffer_pt (cur_bp).p), get_buffer_pt (cur_bp).o + 1) == '\'')))
 
 static int
 move_sexp (int dir)
@@ -787,6 +810,8 @@ move_sexp (int dir)
 
   for (;;)
     {
+      Point pt;
+
       while (dir > 0 ? !eolp () : !bolp ())
         {
           int c = dir > 0 ? following_char () : preceding_char ();
@@ -794,7 +819,9 @@ move_sexp (int dir)
           /* Jump quotes that aren't sexp separators. */
           if (dir > 0 ? PRECEDINGQUOTEDQUOTE (c) : FOLLOWINGQUOTEDQUOTE (c))
             {
-              cur_bp->pt.o += dir;
+              pt = get_buffer_pt (cur_bp);
+              pt.o += dir;
+              set_buffer_pt (cur_bp, pt);
               c = 'a';		/* Treat ' and " like word chars. */
             }
 
@@ -830,14 +857,20 @@ move_sexp (int dir)
                 }
             }
 
-          cur_bp->pt.o += dir;
+          pt = get_buffer_pt (cur_bp);
+          pt.o += dir;
+          set_buffer_pt (cur_bp, pt);
 
           if (!ISSEXPCHAR (c))
             {
               if (gotsexp && level == 0)
                 {
                   if (!ISSEXPSEPARATOR (c))
-                    cur_bp->pt.o -= dir;
+                    {
+                      pt = get_buffer_pt (cur_bp);
+                      pt.o -= dir;
+                      set_buffer_pt (cur_bp, pt);
+                    }
                   return true;
                 }
             }
@@ -852,7 +885,9 @@ move_sexp (int dir)
             minibuf_error ("Scan error: \"Unbalanced parentheses\"");
           break;
         }
-      cur_bp->pt.o = dir > 0 ? 0 : astr_len (cur_bp->pt.p->text);
+      pt = get_buffer_pt (cur_bp);
+      pt.o = dir > 0 ? 0 : astr_len (get_line_text (pt.p));
+      set_buffer_pt (cur_bp, pt);
     }
   return false;
 }
@@ -897,28 +932,30 @@ END_DEFUN
 static void
 astr_append_region (astr s)
 {
-  Region r;
+  Region * rp = region_new ();
   char *t;
 
   activate_mark ();
-  calculate_the_region (&r);
+  calculate_the_region (rp);
 
-  t = copy_text_block (r.start.n, r.start.o, r.size);
-  astr_ncat_cstr (s, t, r.size);
+  t = copy_text_block (get_region_start (rp).n, get_region_start (rp).o, get_region_size (rp));
+  astr_ncat_cstr (s, t, get_region_size (rp));
   free (t);
+
+  free (rp);
 }
 
 static bool
 transpose_subr (int (*forward_func) (void), int (*backward_func) (void))
 {
-  Marker *p0 = point_marker (), *p1, *p2;
+  Marker *p0 = point_marker (), *m1, *m2;
   astr as1, as2 = NULL;
 
   /* For transpose-chars. */
   if (forward_func == forward_char && eolp ())
     backward_func ();
   /* For transpose-lines. */
-  if (forward_func == next_line && cur_bp->pt.n == 0)
+  if (forward_func == next_line && get_buffer_pt (cur_bp).n == 0)
     forward_func ();
 
   /* Backward. */
@@ -934,7 +971,7 @@ transpose_subr (int (*forward_func) (void), int (*backward_func) (void))
 
   /* Mark the beginning of first string. */
   set_mark ();
-  p1 = point_marker ();
+  m1 = point_marker ();
 
   /* Check to make sure we can go forwards twice. */
   if (!forward_func () || !forward_func ())
@@ -947,16 +984,16 @@ transpose_subr (int (*forward_func) (void), int (*backward_func) (void))
       else
         {
           pop_mark ();
-          goto_point (p1->pt);
+          goto_point (get_marker_pt (m1));
           minibuf_error ("End of buffer");
 
           free_marker (p0);
-          free_marker (p1);
+          free_marker (m1);
           return false;
         }
     }
 
-  goto_point (p1->pt);
+  goto_point (get_marker_pt (m1));
 
   /* Forward. */
   forward_func ();
@@ -974,7 +1011,7 @@ transpose_subr (int (*forward_func) (void), int (*backward_func) (void))
 
   /* For transpose-lines. */
   if (forward_func == next_line)
-    p2 = point_marker ();
+    m2 = point_marker ();
   else
     {
       /* Mark the end of second string. */
@@ -982,7 +1019,7 @@ transpose_subr (int (*forward_func) (void), int (*backward_func) (void))
 
       /* Backward. */
       backward_func ();
-      p2 = point_marker ();
+      m2 = point_marker ();
 
       /* Save and delete 2nd marked region. */
       as2 = astr_new ();
@@ -991,19 +1028,19 @@ transpose_subr (int (*forward_func) (void), int (*backward_func) (void))
     }
 
   /* Insert the first string. */
-  goto_point (p2->pt);
-  free_marker (p2);
+  goto_point (get_marker_pt (m2));
+  free_marker (m2);
   insert_astr (as1);
   astr_delete (as1);
 
   /* Insert the second string. */
   if (as2)
     {
-      goto_point (p1->pt);
+      goto_point (get_marker_pt (m1));
       insert_astr (as2);
       astr_delete (as2);
     }
-  free_marker (p1);
+  free_marker (m1);
 
   /* Restore mark. */
   pop_mark ();
@@ -1031,10 +1068,10 @@ transpose (int uniarg, int (*forward_func) (void), int (*backward_func) (void))
       backward_func = tmp_func;
       uniarg = -uniarg;
     }
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
   for (uni = 0; ret && uni < uniarg; ++uni)
     ret = transpose_subr (forward_func, backward_func);
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 
   return bool_to_lisp (ret);
 }
@@ -1131,7 +1168,6 @@ Precisely, if point is on line I, move to the start of line I + N.
       FUNCALL (beginning_of_line);
       ok = execute_with_uniarg (false, n, next_line, previous_line);
     }
-  INT_FREE (n);
 }
 END_DEFUN
 
@@ -1208,15 +1244,15 @@ Fill paragraph at or after point.
   int i, start, end;
   Marker *m = point_marker ();
 
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 
   FUNCALL (forward_paragraph);
-  end = cur_bp->pt.n;
+  end = get_buffer_pt (cur_bp).n;
   if (is_empty_line ())
     end--;
 
   FUNCALL (backward_paragraph);
-  start = cur_bp->pt.n;
+  start = get_buffer_pt (cur_bp).n;
   if (is_empty_line ())
     { /* Move to next line if between two paragraphs. */
       next_line ();
@@ -1237,61 +1273,42 @@ Fill paragraph at or after point.
 
   thisflag &= ~FLAG_DONE_CPCN;
 
-  cur_bp->pt = m->pt;
+  set_buffer_pt (cur_bp, get_marker_pt (m));
   free_marker (m);
 
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 }
 END_DEFUN
 
-/* FIXME: Use astr_recase. */
 static int
 setcase_word (int rcase)
 {
-  int gotword;
-  size_t i, size;
+  size_t i;
   char c;
+  astr as;
 
   if (!ISWORDCHAR (following_char ()))
     if (!forward_word () || !backward_word ())
       return false;
 
-  for (i = cur_bp->pt.o;
-       i < astr_len (cur_bp->pt.p->text) &&
-         ISWORDCHAR ((int) *astr_char (cur_bp->pt.p->text, i));
+  as = astr_new ();
+  for (i = get_buffer_pt (cur_bp).o;
+       i < astr_len (get_line_text (get_buffer_pt (cur_bp).p)) &&
+         ISWORDCHAR ((int) (c = astr_get (get_line_text (get_buffer_pt (cur_bp).p), i)));
        i++)
-    ;
-  size = i - cur_bp->pt.o;
-  if (size > 0)
-    undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, size, size);
+    astr_cat_char (as, c);
 
-  gotword = false;
-  for (gotword = false;
-       cur_bp->pt.o < astr_len (cur_bp->pt.p->text) &&
-         ISWORDCHAR ((c = *astr_char (cur_bp->pt.p->text, cur_bp->pt.o)));
-       cur_bp->pt.o++, gotword = true)
+  if (astr_len (as) > 0)
     {
-      if (isalpha ((int) c))
-        {
-          switch (rcase)
-            {
-            case UPPERCASE:
-              c = toupper (c);
-              break;
-            case LOWERCASE:
-              c = tolower (c);
-              break;
-            case CAPITALIZED:
-              c = (gotword ? tolower : toupper) (c);
-              break;
-            default:
-              break;
-            }
-          astr_nreplace_cstr (cur_bp->pt.p->text, cur_bp->pt.o, 1, &c, 1);
-        }
+      undo_save (UNDO_REPLACE_BLOCK, get_buffer_pt (cur_bp), astr_len (as), astr_len (as));
+      astr_recase (as, rcase);
+      astr_nreplace_cstr (get_line_text (get_buffer_pt (cur_bp).p), get_buffer_pt (cur_bp).o, astr_len (as),
+                          astr_cstr (as), astr_len (as));
     }
+  astr_delete (as);
 
-  cur_bp->flags |= BFLAG_MODIFIED;
+  forward_word ();
+  set_buffer_modified (cur_bp, true);
 
   return true;
 }
@@ -1299,7 +1316,7 @@ setcase_word (int rcase)
 static int
 setcase_word_lowercase (void)
 {
-  return setcase_word (LOWERCASE);
+  return setcase_word (case_lower);
 }
 
 DEFUN ("downcase-word", downcase_word)
@@ -1314,7 +1331,7 @@ END_DEFUN
 static int
 setcase_word_uppercase (void)
 {
-  return setcase_word (UPPERCASE);
+  return setcase_word (case_upper);
 }
 
 DEFUN ("upcase-word", upcase_word)
@@ -1329,7 +1346,7 @@ END_DEFUN
 static int
 setcase_word_capitalize (void)
 {
-  return setcase_word (CAPITALIZED);
+  return setcase_word (case_capitalized);
 }
 
 DEFUN ("capitalize-word", capitalize_word)
@@ -1345,39 +1362,43 @@ END_DEFUN
 
 /*
  * Set the region case.
- * FIXME: Once regions are just strings, use astr_recase.
  */
 static le *
-setcase_region (int rcase)
+setcase_region (enum casing rcase)
 {
-  Region r;
-  Line *lp;
-  size_t i;
-  int (*func) (int) = rcase == UPPERCASE ? toupper : tolower;
+  Region * rp = region_new ();
+  Line * lp;
+  size_t i, size;
+  int (*func) (int) = rcase == case_upper ? toupper : tolower;
 
-  if (warn_if_readonly_buffer () || !calculate_the_region (&r))
-    return leNIL;
-
-  undo_save (UNDO_REPLACE_BLOCK, r.start, r.size, r.size);
-
-  lp = r.start.p;
-  i = r.start.o;
-  while (r.size--)
+  if (warn_if_readonly_buffer () || !calculate_the_region (rp))
     {
-      if (i < astr_len (lp->text))
+      free (rp);
+      return leNIL;
+    }
+
+  size = get_region_size (rp);
+  undo_save (UNDO_REPLACE_BLOCK, get_region_start (rp), size, get_region_size (rp));
+
+  lp = get_region_start (rp).p;
+  i = get_region_start (rp).o;
+  while (size--)
+    {
+      if (i < astr_len (get_line_text (lp)))
         {
-          char c = func (*astr_char (lp->text, i));
-          astr_nreplace_cstr (lp->text, i, 1, &c, 1);
+          char c = func (astr_get (get_line_text (lp), i));
+          astr_nreplace_cstr (get_line_text (lp), i, 1, &c, 1);
           ++i;
         }
       else
         {
-          lp = lp->next;
+          lp = get_line_next (lp);
           i = 0;
         }
     }
 
-  cur_bp->flags |= BFLAG_MODIFIED;
+  set_buffer_modified (cur_bp, true);
+  free (rp);
 
   return leT;
 }
@@ -1387,7 +1408,7 @@ DEFUN ("upcase-region", upcase_region)
 Convert the region to upper case.
 +*/
 {
-  ok = setcase_region (UPPERCASE);
+  ok = setcase_region (case_upper);
 }
 END_DEFUN
 
@@ -1396,7 +1417,7 @@ DEFUN ("downcase-region", downcase_region)
 Convert the region to lower case.
 +*/
 {
-  ok = setcase_region (LOWERCASE);
+  ok = setcase_region (case_lower);
 }
 END_DEFUN
 
@@ -1411,7 +1432,7 @@ write_shell_output (va_list ap)
 static bool
 pipe_command (const char *cmd, const char *tempfile, bool insert, bool replace)
 {
-  astr out = astr_new ();
+  astr out;
   bool more_than_one_line = false;
   char *cmdline, *eol;
   FILE * pipe;
@@ -1439,12 +1460,12 @@ pipe_command (const char *cmd, const char *tempfile, bool insert, bool replace)
         {
           if (replace)
             {
-              undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+              undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
               FUNCALL (delete_region);
             }
             insert_astr (out);
             if (replace)
-              undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+              undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
         }
       else
         {
@@ -1506,7 +1527,6 @@ says to insert the output in the current buffer.
     ok = bool_to_lisp (pipe_command (cmd, "/dev/null", insert, false));
 
   STR_FREE (cmd);
-  BOOL_FREE (insert);
 }
 END_DEFUN
 
@@ -1534,9 +1554,9 @@ The output is available in that buffer in both cases.
 
   if (cmd != NULL)
     {
-      Region r;
+      Region * rp = region_new ();
 
-      if (!calculate_the_region (&r))
+      if (!calculate_the_region (rp))
         ok = leNIL;
       else
         {
@@ -1550,13 +1570,13 @@ The output is available in that buffer in both cases.
             }
           else
             {
-              char *p = copy_text_block (r.start.n, r.start.o, r.size);
-              ssize_t written = write (fd, p, r.size);
+              char *p = copy_text_block (get_region_start (rp).n, get_region_start (rp).o, get_region_size (rp));
+              ssize_t written = write (fd, p, get_region_size (rp));
 
               free (p);
               close (fd);
 
-              if (written != (ssize_t) r.size)
+              if (written != (ssize_t) get_region_size (rp))
                 {
                   if (written == -1)
                     minibuf_error ("Error writing to temporary file: %s",
@@ -1572,45 +1592,29 @@ The output is available in that buffer in both cases.
 
           remove (tempfile);
         }
+
+      free (rp);
     }
 
   STR_FREE (cmd);
 }
 END_DEFUN
 
-bool
-delete_region (const Region * r)
-{
-  size_t size = r->size;
-
-  if (warn_if_readonly_buffer ())
-    return false;
-
-  if (cur_bp->pt.p != r->start.p || r->start.o != cur_bp->pt.o)
-    FUNCALL (exchange_point_and_mark);
-
-  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, size, 0);
-  undo_nosave = true;
-  while (size--)
-    FUNCALL (delete_char);
-  undo_nosave = false;
-
-  return true;
-}
-
 DEFUN ("delete-region", delete_region)
 /*+
 Delete the text between point and mark.
 +*/
 {
-  Region r;
+  Region * rp = region_new ();
 
-  if (!calculate_the_region (&r))
+  if (!calculate_the_region (rp))
     ok = leNIL;
-  else if (!delete_region (&r))
+  else if (!delete_region (rp))
     ok = leNIL;
   else
     deactivate_mark ();
+
+  free (rp);
 }
 END_DEFUN
 
@@ -1621,7 +1625,7 @@ On isolated blank line, delete that one.
 On nonblank line, delete any immediately following blank lines.
 +*/
 {
-  Marker *old_marker = point_marker ();
+  Marker *m = point_marker ();
   int seq_started = false;
 
   /* Delete any immediately following blank lines.  */
@@ -1638,7 +1642,7 @@ On nonblank line, delete any immediately following blank lines.
           if (!seq_started)
             {
               seq_started = true;
-              undo_save (UNDO_START_SEQUENCE, old_marker->pt, 0, 0);
+              undo_save (UNDO_START_SEQUENCE, get_marker_pt (m), 0, 0);
             }
           FUNCALL (delete_region);
           pop_mark ();
@@ -1665,12 +1669,12 @@ On nonblank line, delete any immediately following blank lines.
       while (is_blank_line ());
       if (forward)
         FUNCALL (forward_line);
-      if (cur_bp->pt.p != old_marker->pt.p)
+      if (get_buffer_pt (cur_bp).p != get_marker_pt (m).p)
         {
           if (!seq_started)
             {
               seq_started = true;
-              undo_save (UNDO_START_SEQUENCE, old_marker->pt, 0, 0);
+              undo_save (UNDO_START_SEQUENCE, get_marker_pt (m), 0, 0);
             }
           FUNCALL (delete_region);
         }
@@ -1690,12 +1694,12 @@ On nonblank line, delete any immediately following blank lines.
       pop_mark ();
     }
 
-  cur_bp->pt = old_marker->pt;
+  set_buffer_pt (cur_bp, get_marker_pt (m));
 
   if (seq_started)
-    undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+    undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
 
-  free_marker (old_marker);
+  free_marker (m);
   deactivate_mark ();
 }
 END_DEFUN
