@@ -57,21 +57,21 @@ kill_ring_push_nstring (char *s, size_t size)
 }
 
 static bool
-copy_or_kill_region (bool kill, Region * r)
+copy_or_kill_region (bool kill, Region * rp)
 {
-  char *p = copy_text_block (r->start.n, r->start.o, r->size);
+  char *p = copy_text_block (get_region_start (rp).n, get_region_start (rp).o, get_region_size (rp));
 
   if (!(lastflag & FLAG_DONE_KILL))
     free_kill_ring ();
-  kill_ring_push_nstring (p, r->size);
+  kill_ring_push_nstring (p, get_region_size (rp));
   free (p);
 
   if (kill)
     {
-      if (cur_bp->flags & BFLAG_READONLY)
+      if (get_buffer_readonly (cur_bp))
         minibuf_error ("Read only text copied to kill ring");
       else
-        assert (delete_region (r));
+        assert (delete_region (rp));
     }
 
   thisflag |= FLAG_DONE_KILL;
@@ -91,12 +91,16 @@ kill_line (int literally)
 
   if (!eolp ())
     {
-      Region r;
+      Region * rp = region_new ();
+      bool ret;
 
-      r.start = cur_bp->pt;
-      r.size = astr_len (cur_bp->pt.p->text) - cur_bp->pt.o;
+      set_region_start (rp, get_buffer_pt (cur_bp));
+      set_region_size (rp, astr_len (get_line_text (get_buffer_pt (cur_bp).p)) - get_buffer_pt (cur_bp).o);
 
-      if (!copy_or_kill_region (true, &r))
+      ret = copy_or_kill_region (true, rp);
+      free (rp);
+
+      if (!ret)
         return false;
 
       if (!literally)
@@ -144,12 +148,14 @@ END_DEFUN
 static bool
 copy_or_kill_the_region (bool kill)
 {
-  Region r;
+  Region * rp = region_new ();
+  bool ret = false;
 
-  if (!calculate_the_region (&r))
-    return false;
+  if (calculate_the_region (rp))
+    ret = copy_or_kill_region (kill, rp);
 
-  return copy_or_kill_region (kill, &r);
+  free (rp);
+  return ret;
 }
 
 DEFUN ("kill-region", kill_region)
@@ -188,10 +194,10 @@ kill (int uniarg, Function mark_func)
     return leNIL;
 
   push_mark ();
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
   mark_func (uniarg, NULL);
   FUNCALL (kill_region);
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  undo_save (UNDO_END_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
   pop_mark ();
 
   thisflag |= FLAG_DONE_KILL;
@@ -248,7 +254,8 @@ killed @i{or} yanked.  Put point at end, and set mark at beginning.
 
   set_mark_interactive ();
 
-  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, astr_len (kill_ring_text));
+  undo_save (UNDO_REPLACE_BLOCK, get_buffer_pt (cur_bp), 0,
+             astr_len (kill_ring_text));
   undo_nosave = true;
   insert_astr (kill_ring_text);
   undo_nosave = false;
