@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <limits.h>
+#include "config.h"
 #include "xalloc.h"
 #include "size_max.h"
 #include "minmax.h"
@@ -41,30 +42,19 @@
  *--------------------------------------------------------------------------*/
 
 /* Opaque types. */
+typedef struct Line Line;
+typedef struct Region Region;
+typedef struct Marker Marker;
 typedef struct History History;
 typedef struct Undo Undo;
 typedef struct Macro Macro;
 typedef struct Binding *Binding;
-
-/* FIXME: Types which should really be opaque. */
-typedef struct Line Line;
-typedef struct Point Point;
-typedef struct Marker Marker;
-typedef struct Region Region;
 typedef struct Buffer Buffer;
 typedef struct Window Window;
 typedef struct Completion Completion;
 
-/*
- * A line is a doubly-linked list of astrs.
- * The newline at the end of each line is implicit.
- */
-struct Line
-{
-  Line *prev;
-  Line *next;
-  astr text;
-};
+/* FIXME: Types which should really be opaque. */
+typedef struct Point Point;
 
 struct Point
 {
@@ -73,71 +63,12 @@ struct Point
   size_t o;			/* Offset. */
 };
 
-struct Marker
-{
-  Buffer *bp;			/* Buffer that points into. */
-  Point pt;			/* Point position. */
-  Marker *next;			/* Used to chain all markers in the buffer. */
-};
-
 /* Undo delta types. */
 enum
 {
   UNDO_REPLACE_BLOCK,		/* Replace a block of characters. */
   UNDO_START_SEQUENCE,		/* Start a multi operation sequence. */
-  UNDO_END_SEQUENCE,		/* End a multi operation sequence. */
-};
-
-struct Region
-{
-  Point start;			/* The region start. */
-  Point end;			/* The region end. */
-  size_t size;			/* The region size. */
-};
-
-/* Buffer flags or minor modes. */
-#define BFLAG_MODIFIED  0000001	/* The buffer has been modified. */
-#define BFLAG_NOSAVE    0000002	/* The buffer need not to be saved. */
-#define BFLAG_NEEDNAME  0000004	/* On save, ask for a file name. */
-#define BFLAG_TEMPORARY 0000010	/* The buffer is a temporary buffer. */
-#define BFLAG_READONLY  0000020	/* The buffer cannot be modified. */
-#define BFLAG_OVERWRITE 0000040	/* The buffer is in overwrite mode. */
-#define BFLAG_BACKUP    0000100	/* The old file has already been
-                                   backed up. */
-#define BFLAG_NOUNDO    0000200	/* Do not record undo informations. */
-#define BFLAG_AUTOFILL  0000400	/* The buffer is in Auto Fill mode. */
-#define BFLAG_ISEARCH   0001000	/* The buffer is in Isearch loop. */
-#define BFLAG_MARK      0002000	/* The mark is active. */
-
-struct Buffer
-{
-  Buffer *next;		/* Next buffer in buffer list. */
-  char *name;		/* The name of the buffer. */
-  char *filename;	/* The file being edited. */
-  char *eol;		/* EOL string (up to 2 chars). */
-  Line *lines;		/* The lines of text. */
-  size_t last_line;	/* The number of the last line in the buffer. */
-  Point pt;		/* The point. */
-  Marker *mark;		/* The mark. */
-  Marker *markers;	/* Markers list (updated whenever text is changed). */
-  int flags;		/* Buffer flags. */
-  Undo *last_undop;	/* Most recent undo delta. */
-  Undo *next_undop;	/* Next undo delta to apply. */
-  Hash_table *vars;	/* Buffer-local variables. */
-};
-
-struct Window
-{
-  Window *next;		/* The next window in window list. */
-  Buffer *bp;		/* The buffer displayed in window. */
-  size_t topdelta;	/* The top line delta. */
-  int lastpointn;	/* The last point line number. */
-  size_t start_column;	/* The start column of the window (>0 if scrolled
-                           sideways). */
-  Marker *saved_pt;	/* The point line pointer, line number and offset
-                           (used to hold the point in non-current windows). */
-  size_t fwidth, fheight; /* The formal width and height of the window. */
-  size_t ewidth, eheight; /* The effective width and height of the window. */
+  UNDO_END_SEQUENCE		/* End a multi operation sequence. */
 };
 
 enum
@@ -148,23 +79,37 @@ enum
   COMPLETION_NONUNIQUE
 };
 
-struct Completion
-{
-  Buffer *old_bp;		/* The buffer from which the
-                                   completion was invoked. */
-  gl_list_t completions;	/* The completions list. */
-  gl_list_t matches;		/* The matches list. */
-  const char *match;		/* The match buffer. */
-  size_t matchsize;		/* The match buffer size. */
-  int flags;			/* Completion flags. */
-  astr path;			/* Path for a filename completion. */
-};
-
 /* Completion flags */
 #define CFLAG_POPPEDUP	0000001	/* Completion window has been popped up. */
 #define CFLAG_CLOSE	0000002	/* The completion window should be closed. */
 #define CFLAG_FILENAME	0000004	/* This is a filename completion. */
 
+/*--------------------------------------------------------------------------
+ * Object field getter and setter generators.
+ *--------------------------------------------------------------------------*/
+
+#define GETTER(Obj, name, ty, field)            \
+  ty                                            \
+  get_ ## name ## _ ## field (const Obj *p)     \
+  {                                             \
+    return p->field;                            \
+  }                                             \
+
+#define SETTER(Obj, name, ty, field)            \
+  void                                          \
+  set_ ## name ## _ ## field (Obj *p, ty field) \
+  {                                             \
+    p->field = field;                           \
+  }
+
+#define STR_SETTER(Obj, name, field)                            \
+  void                                                          \
+  set_ ## name ## _ ## field (Obj *p, const char *field)        \
+  {                                                             \
+    free ((char *) p->field);                                   \
+    p->field = field ? xstrdup (field) : NULL;                  \
+  }
+
 /*--------------------------------------------------------------------------
  * Zile commands to C bindings.
  *--------------------------------------------------------------------------*/
@@ -191,9 +136,9 @@ typedef le * (*Function) (long uniarg, le * list);
         }
 
 /* Define a non-user-visible function. */
-#define DEFUN_HIDDEN(zile_func, c_func) \
+#define DEFUN_NONINTERACTIVE(zile_func, c_func) \
         DEFUN(zile_func, c_func)
-#define DEFUN_HIDDEN_ARGS(zile_func, c_func, args) \
+#define DEFUN_NONINTERACTIVE_ARGS(zile_func, c_func, args) \
         DEFUN_ARGS(zile_func, c_func, args)
 
 /* String argument. */
@@ -223,7 +168,6 @@ typedef le * (*Function) (long uniarg, le * list);
             if (name == LONG_MAX) \
               ok = leNIL; \
           }
-#define INT_FREE(name)
 
 /* Boolean argument. */
 #define BOOL_ARG(name) \
@@ -236,7 +180,6 @@ typedef le * (*Function) (long uniarg, le * list);
             if (strcmp (s, "nil") == 0) \
               name = false; \
           }
-#define BOOL_FREE(name)
 
 /* Call an interactive function. */
 #define FUNCALL(c_func)                         \
@@ -293,18 +236,20 @@ typedef le * (*Function) (long uniarg, le * list);
  *--------------------------------------------------------------------------*/
 
 /* Global flags, stored in thisflag and lastflag. */
-#define FLAG_DONE_CPCN		0000001	/* Last command was C-p or C-n. */
-#define FLAG_DONE_KILL		0000002	/* The last command was a kill. */
-#define FLAG_NEED_RESYNC	0000004	/* A resync is required. */
-#define FLAG_QUIT		0000010	/* The user has asked to quit. */
-#define FLAG_SET_UNIARG		0000020	/* The last command modified the
+#define FLAG_DONE_CPCN		0001	/* Last command was C-p or C-n. */
+#define FLAG_DONE_KILL		0002	/* The last command was a kill. */
+#define FLAG_NEED_RESYNC	0004	/* A resync is required. */
+#define FLAG_QUIT		0010	/* The user has asked to quit. */
+#define FLAG_SET_UNIARG		0020	/* The last command modified the
                                            universal arg variable `uniarg'. */
-#define FLAG_UNIARG_EMPTY	0000040	/* Current universal arg is just C-u's
+#define FLAG_UNIARG_EMPTY	0040	/* Current universal arg is just C-u's
                                            with no number. */
-#define FLAG_DEFINING_MACRO	0000100	/* We are defining a macro. */
+#define FLAG_DEFINING_MACRO	0100	/* We are defining a macro. */
 
-/* Zile font codes
- * Designed to fit in an int, leaving room for a char underneath. */
+/*
+ * Zile font codes
+ * Designed to fit in an int, leaving room for a char underneath.
+ */
 #define FONT_NORMAL		0x000
 #define FONT_REVERSE		0x100
 
