@@ -132,13 +132,9 @@ re_find_substr (const char *s1, size_t s1size,
   struct re_registers search_regs;
   char *ret = NULL;
   int index;
-  reg_syntax_t old_syntax;
-
-  old_syntax = re_set_syntax (RE_SYNTAX_EMACS);
+  reg_syntax_t old_syntax = re_set_syntax (RE_SYNTAX_EMACS);
 
   search_regs.num_regs = 1;
-  search_regs.start = XZALLOC (regoff_t);
-  search_regs.end = XZALLOC (regoff_t);
 
   /* translate table is never written to, so this cast is safe */
   pattern.translate = (unsigned char *) translate;
@@ -182,18 +178,16 @@ re_find_substr (const char *s1, size_t s1size,
                                    or malloced */
   regfree (&pattern);
 
-  free (search_regs.start);
-  free (search_regs.end);
-
   return ret;
 }
 
 static void
 goto_linep (Line * lp)
 {
-  cur_bp->pt = point_min ();
+  Point pt;
+  set_buffer_pt (cur_bp, point_min ());
   resync_redisplay ();
-  while (cur_bp->pt.p != lp)
+  for (pt = get_buffer_pt (cur_bp); pt.p != lp; pt = get_buffer_pt (cur_bp))
     next_line ();
 }
 
@@ -207,32 +201,35 @@ search_forward (Line * startp, size_t starto, const char *s, int regexp)
   if (s2size < 1)
     return false;
 
-  for (lp = startp; lp != cur_bp->lines; lp = lp->next)
+  for (lp = startp; lp != get_buffer_lines (cur_bp); lp = get_line_next (lp))
     {
       if (lp == startp)
         {
-          sp = astr_char (lp->text, starto);
-          s1size = astr_len (lp->text) - starto;
+          sp = astr_cstr (get_line_text (lp)) + starto;
+          s1size = astr_len (get_line_text (lp)) - starto;
         }
       else
         {
-          sp = astr_cstr (lp->text);
-          s1size = astr_len (lp->text);
+          sp = astr_cstr (get_line_text (lp));
+          s1size = astr_len (get_line_text (lp));
         }
       if (s1size < 1)
         continue;
 
       if (regexp)
         sp2 = re_find_substr (sp, s1size, s, s2size,
-                              sp == astr_cstr (lp->text), true, false,
+                              sp == astr_cstr (get_line_text (lp)), true, false,
                               translate);
       else
         sp2 = find_substr (sp, s1size, s, s2size, translate);
 
       if (sp2 != NULL)
         {
+          Point pt;
           goto_linep (lp);
-          cur_bp->pt.o = sp2 - astr_cstr (lp->text);
+          pt = get_buffer_pt (cur_bp);
+          pt.o = sp2 - astr_cstr (get_line_text (lp));
+          set_buffer_pt (cur_bp, pt);
           return true;
         }
     }
@@ -250,27 +247,30 @@ search_backward (Line * startp, size_t starto, const char *s, int regexp)
   if (ssize < 1)
     return false;
 
-  for (lp = startp; lp != cur_bp->lines; lp = lp->prev)
+  for (lp = startp; lp != get_buffer_lines (cur_bp); lp = get_line_prev (lp))
     {
-      sp = astr_cstr (lp->text);
+      sp = astr_cstr (get_line_text (lp));
       if (lp == startp)
         s1size = starto;
       else
-        s1size = astr_len (lp->text);
+        s1size = astr_len (get_line_text (lp));
       if (s1size < 1)
         continue;
 
       if (regexp)
         sp2 = re_find_substr (sp, s1size, s, ssize,
-                              true, s1size == astr_len (lp->text), true,
+                              true, s1size == astr_len (get_line_text (lp)), true,
                               translate);
       else
         sp2 = rfind_substr (sp, s1size, s, ssize, translate);
 
       if (sp2 != NULL)
         {
+          Point pt;
           goto_linep (lp);
-          cur_bp->pt.o = sp2 - astr_cstr (lp->text);
+          pt = get_buffer_pt (cur_bp);
+          pt.o = sp2 - astr_cstr (get_line_text (lp));
+          set_buffer_pt (cur_bp, pt);
           return true;
         }
     }
@@ -285,29 +285,30 @@ typedef int (*Searcher) (Line * startp, size_t starto, const char *s, int regexp
 static le *
 search (Searcher searcher, bool regexp, const char *pattern, const char *search_msg)
 {
+  le * ok = leT;
+  const char *ms = NULL;
+
   if (pattern == NULL)
-    pattern = minibuf_read (search_msg, last_search);
+    pattern = ms = minibuf_read (search_msg, last_search);
 
   if (pattern == NULL)
     return FUNCALL (keyboard_quit);
   if (pattern[0] == '\0')
+    ok = leNIL;
+  else
     {
-      free ((char *) pattern);
-      return leNIL;
+      free (last_search);
+      last_search = xstrdup (pattern);
+
+      if (!searcher (get_buffer_pt (cur_bp).p, get_buffer_pt (cur_bp).o, pattern, regexp))
+        {
+          minibuf_error ("Search failed: \"%s\"", pattern);
+          ok = leNIL;
+        }
     }
 
-  free (last_search);
-  last_search = xstrdup (pattern);
-
-  if (!searcher (cur_bp->pt.p, cur_bp->pt.o, pattern, regexp))
-    {
-      minibuf_error ("Search failed: \"%s\"", pattern);
-      free ((char *) pattern);
-      return leNIL;
-    }
-
-  free ((char *) pattern);
-  return leT;
+  free ((char *) ms);
+  return ok;
 }
 
 DEFUN_ARGS ("search-forward", search_forward,
@@ -371,13 +372,12 @@ isearch (int dir, int regexp)
   astr buf = astr_new ();
   astr pattern = astr_new ();
   Point start, cur;
-  Marker *old_mark = cur_wp->bp->mark ? copy_marker (cur_wp->bp->mark) : NULL;
+  Marker *old_mark = copy_marker (get_buffer_mark (get_window_bp (cur_wp)));
 
-  start = cur_bp->pt;
-  cur = cur_bp->pt;
+  start = cur = get_buffer_pt (cur_bp);
 
   /* I-search mode. */
-  cur_wp->bp->flags |= BFLAG_ISEARCH;
+  set_buffer_isearch (get_window_bp (cur_wp), true);
 
   for (;;)
     {
@@ -409,20 +409,17 @@ isearch (int dir, int regexp)
 
       if (c == KBD_CANCEL)
         {
-          cur_bp->pt = start;
+          set_buffer_pt (cur_bp, start);
           thisflag |= FLAG_NEED_RESYNC;
 
           /* Quit. */
           FUNCALL (keyboard_quit);
 
           /* Restore old mark position. */
-          if (cur_bp->mark)
-            free_marker (cur_bp->mark);
+          if (get_buffer_mark (cur_bp))
+            free_marker (get_buffer_mark (cur_bp));
 
-          if (old_mark)
-            cur_bp->mark = copy_marker (old_mark);
-          else
-            cur_bp->mark = old_mark;
+          set_buffer_mark (cur_bp, copy_marker (old_mark));
           break;
         }
       else if (c == KBD_BS)
@@ -430,7 +427,8 @@ isearch (int dir, int regexp)
           if (astr_len (pattern) > 0)
             {
               astr_truncate (pattern, astr_len (pattern) - 1);
-              cur = cur_bp->pt = start;
+              cur = start;
+              set_buffer_pt (cur_bp, start);
               thisflag |= FLAG_NEED_RESYNC;
             }
           else
@@ -452,7 +450,7 @@ isearch (int dir, int regexp)
           if (astr_len (pattern) > 0)
             {
               /* Find next match. */
-              cur = cur_bp->pt;
+              cur = get_buffer_pt (cur_bp);
               /* Save search string. */
               free (last_search);
               last_search = xstrdup (astr_cstr (pattern));
@@ -485,7 +483,7 @@ isearch (int dir, int regexp)
                 {
                   /* Save mark. */
                   set_mark ();
-                  cur_bp->mark->pt = start;
+                  set_marker_pt (get_buffer_mark (cur_bp), start);
 
                   /* Save search string. */
                   free (last_search);
@@ -519,7 +517,7 @@ isearch (int dir, int regexp)
     }
 
   /* done */
-  cur_wp->bp->flags &= ~BFLAG_ISEARCH;
+  set_buffer_isearch (get_window_bp (cur_wp), false);
 
   astr_delete (buf);
   astr_delete (pattern);
@@ -612,7 +610,7 @@ what to do with it.
     }
 
   /* Spaghetti code follows... :-( */
-  while (search_forward (cur_bp->pt.p, cur_bp->pt.o, find, false))
+  while (search_forward (get_buffer_pt (cur_bp).p, get_buffer_pt (cur_bp).o, find, false))
     {
       if (!noask)
         {
@@ -658,12 +656,14 @@ what to do with it.
         }
 
     replblock:
-      ++count;
-      undo_save (UNDO_REPLACE_BLOCK,
-                 make_point (cur_bp->pt.n,
-                             cur_bp->pt.o - find_len), find_len, strlen (repl));
-      line_replace_text (&cur_bp->pt.p, cur_bp->pt.o - find_len,
-                         find_len, repl, find_no_upper);
+      {
+        Point pt = get_buffer_pt (cur_bp);
+        ++count;
+        undo_save (UNDO_REPLACE_BLOCK,
+                   make_point (pt.n, pt.o - find_len), find_len, strlen (repl));
+        line_replace_text (&pt.p, pt.o - find_len, find_len, repl, find_no_upper);
+        set_buffer_pt (cur_bp, pt);
+      }
     nextmatch:
       if (exitloop)
         break;
@@ -680,3 +680,8 @@ endoffunc:
   minibuf_write ("Replaced %d occurrences", count);
 }
 END_DEFUN
+
+void free_search (void)
+{
+  free (last_search);
+}
