@@ -184,7 +184,7 @@ do_minibuf_read (const char *prompt, const char *value, size_t pos,
               break;
             }
 
-          if (cp->flags & CFLAG_POPPEDUP)
+          if (get_completion_flags (cp) & CFLAG_POPPEDUP)
             {
               completion_scroll_down ();
               thistab = lasttab;
@@ -198,7 +198,7 @@ do_minibuf_read (const char *prompt, const char *value, size_t pos,
               break;
             }
 
-          if (cp->flags & CFLAG_POPPEDUP)
+          if (get_completion_flags (cp) & CFLAG_POPPEDUP)
             {
               completion_scroll_up ();
               thistab = lasttab;
@@ -242,7 +242,7 @@ do_minibuf_read (const char *prompt, const char *value, size_t pos,
             }
 
           if (lasttab != -1 && lasttab != COMPLETION_NOTMATCHED
-              && cp->flags & CFLAG_POPPEDUP)
+              && get_completion_flags (cp) & CFLAG_POPPEDUP)
             {
               completion_scroll_up ();
               thistab = lasttab;
@@ -260,9 +260,9 @@ do_minibuf_read (const char *prompt, const char *value, size_t pos,
                 case COMPLETION_NONUNIQUE:
                   {
                     bs = astr_new ();
-                    if (cp->flags & CFLAG_FILENAME)
-                      astr_cat (bs, cp->path);
-                    astr_ncat_cstr (bs, cp->match, cp->matchsize);
+                    if (get_completion_flags (cp) & CFLAG_FILENAME)
+                      astr_cat (bs, get_completion_path (cp));
+                    astr_ncat_cstr (bs, get_completion_match (cp), get_completion_matchsize (cp));
                     if (strncmp (astr_cstr (as), astr_cstr (bs),
                                  astr_len (bs)) != 0)
                       thistab = -1;
@@ -313,14 +313,14 @@ term_minibuf_read (const char *prompt, const char *value, size_t pos,
       astr_delete (as);
     }
 
-  if (cp != NULL && (cp->flags & CFLAG_POPPEDUP)
+  if (cp != NULL && (get_completion_flags (cp) & CFLAG_POPPEDUP)
       && (wp = find_window ("*Completions*")) != NULL)
     {
       set_current_window (wp);
-      if (cp->flags & CFLAG_CLOSE)
+      if (get_completion_flags (cp) & CFLAG_CLOSE)
         FUNCALL (delete_window);
-      else if (cp->old_bp)
-        switch_to_buffer (cp->old_bp);
+      else if (get_completion_old_bp (cp))
+        switch_to_buffer (get_completion_old_bp (cp));
       set_current_window (old_wp);
     }
 
